import streamlit as st
import sys
import os

try:
    from alphastats.gui.utils.ui_helper import sidebar_info
    from alphastats.gui.utils.analysis_helper import *
    from alphastats.gui.utils.software_options import software_options
    from alphastats.loader.MaxQuantLoader import MaxQuantLoader
    from alphastats.DataSet import DataSet

except ModuleNotFoundError:
    from utils.ui_helper import sidebar_info
    from utils.analysis_helper import *
    from utils.software_options import software_options
    from alphastats import MaxQuantLoader
    from alphastats import DataSet



import pandas as pd
import plotly.express as px

from streamlit.runtime import get_instance
from streamlit.runtime.scriptrunner.script_run_context import get_script_run_ctx

runtime = get_instance()
session_id = get_script_run_ctx().session_id
session_info = runtime._session_mgr.get_session_info(session_id)

user_session_id = session_id
st.session_state["user_session_id"] = user_session_id


def load_options():

    from alphastats.gui.utils.options import plotting_options, statistic_options

    st.session_state["plotting_options"] = plotting_options
    st.session_state["statistic_options"] = statistic_options


def check_software_file(df, software):
    """
    check if software files are in right format
    can be fragile when different settings are used or software is updated
    """

    if software == "MaxQuant":
        expected_columns = ["Protein IDs", "Reverse", "Potential contaminant"]
        if (set(expected_columns).issubset(set(df.columns.to_list()))) == False:
            st.error(
                "This is not a valid MaxQuant file. Please check:"
                "http://www.coxdocs.org/doku.php?id=maxquant:table:proteingrouptable"
            )

    elif software == "AlphaPept":
        if "object" in df.iloc[:, 1:].dtypes.to_list():
            st.error("This is not a valid AlphaPept file.")

    elif software == "DIANN":
        expected_columns = [
            "Protein.Group",
        ]

        if (set(expected_columns).issubset(set(df.columns.to_list()))) == False:
            st.error("This is not a valid DIA-NN file.")

    elif software == "Spectronaut":
        expected_columns = [
            "PG.ProteinGroups",
        ]

        if (set(expected_columns).issubset(set(df.columns.to_list()))) == False:
            st.error("This is not a valid Spectronaut file.")

    elif software == "FragPipe":
        expected_columns = ["Protein"]
        if (set(expected_columns).issubset(set(df.columns.to_list()))) == False:
            st.error(
                "This is not a valid FragPipe file. Please check:"
                "https://fragpipe.nesvilab.org/docs/tutorial_fragpipe_outputs.html#combined_proteintsv"
            )


def print_software_import_info(software):
    import_file = software_options.get(software).get("import_file")
    string_output = f"Please upload {import_file} file from {software}."
    return string_output


def select_columns_for_loaders(software):
    """
    select intensity and index column depending on software
    will be saved in session state
    """
    st.write("\n\n")
    st.markdown("### 2. Select columns used for further analysis.")
    st.markdown("Select intensity columns for further analysis")

    st.selectbox(
        "Intensity Column",
        options=software_options.get(software).get("intensity_column"),
        key="intensity_column",
    )

    st.markdown("Select index column (with ProteinGroups) for further analysis")

    st.selectbox(
        "Index Column",
        options=software_options.get(software).get("index_column"),
        key="index_column",
    )


def load_proteomics_data(uploaded_file, intensity_column, index_column, software):
    """load software file into loader object from alphastats"""
    loader = software_options.get(software)["loader_function"](
        uploaded_file, intensity_column, index_column
    )
    return loader


def select_sample_column_metadata(df, software):
    samples_proteomics_data = get_sample_names_from_software_file()
    valid_sample_columns = []

    for col in df.columns.to_list():
        if bool(set(samples_proteomics_data) & set(df[col].to_list())):
            valid_sample_columns.append(col)

    if len(valid_sample_columns) == 0:
        st.error(
            f"Metadata does not match Proteomics data."
            f"Information for the samples: {samples_proteomics_data} is required."
        )

    st.write(
        f"Select column that contains sample IDs matching the sample names described"
        + f"in {software_options.get(software).get('import_file')}"
    )

    with st.form("sample_column"):
        st.selectbox("Sample Column", options=valid_sample_columns, key="sample_column")
        submitted = st.form_submit_button("Create DataSet")

    if submitted:
        return True


def upload_softwarefile(software):

    softwarefile = st.file_uploader(
        print_software_import_info(software=software),
        type=["csv", "tsv", "txt", "hdf"],
    )

    if softwarefile is not None:

        softwarefile_df = read_uploaded_file_into_df(softwarefile)
        # display head a protein data

        check_software_file(softwarefile_df, software)

        st.write(
            f"File successfully uploaded. Number of rows: {softwarefile_df.shape[0]}"
            f", Number of columns: {softwarefile_df.shape[1]}.\nPreview:"
        )
        st.dataframe(softwarefile_df.head(5))

        select_columns_for_loaders(software=software)

        if (
            "intensity_column" in st.session_state
            and "index_column" in st.session_state
        ):
            loader = load_proteomics_data(
                softwarefile_df,
                intensity_column=st.session_state.intensity_column,
                index_column=st.session_state.index_column,
                software=software,
            )
            st.session_state["loader"] = loader


def upload_metadatafile(software):

    st.write("\n\n")
    st.markdown("### 3. Upload corresponding metadata.")
    st.file_uploader(
        "Upload metadata file. with information about your samples",
        key="metadatafile",
    )

    if st.session_state.metadatafile is not None:

        metadatafile_df = read_uploaded_file_into_df(st.session_state.metadatafile)
        # display metadata
        st.write(
            f"File successfully uploaded. Number of rows: {metadatafile_df.shape[0]}"
            f", Number of columns: {metadatafile_df.shape[1]}. \nPreview:"
        )
        st.dataframe(metadatafile_df.head(5))
        # pick sample column

        if select_sample_column_metadata(metadatafile_df, software):
            # create dataset
            st.session_state["dataset"] = DataSet(
                loader=st.session_state.loader,
                metadata_path=metadatafile_df,
                sample_column=st.session_state.sample_column,
            )
            st.session_state["metadata_columns"] = metadatafile_df.columns.to_list()

            load_options()

            display_loaded_dataset()

    if st.button("Create a DataSet without metadata"):
        st.session_state["dataset"] = DataSet(loader=st.session_state.loader)
        st.session_state["metadata_columns"] = ["sample"]

        load_options()

        display_loaded_dataset()


def load_sample_data():
    _this_file = os.path.abspath(__file__)
    _this_directory = os.path.dirname(_this_file)
    _parent_directory = os.path.dirname(_this_directory)     
    folder_to_load = os.path.join(_parent_directory, 'sample_data')
    
    filepath= os.path.join(folder_to_load, "proteinGroups.txt")
    metadatapath= os.path.join(folder_to_load, "metadata.xlsx")


    loader = MaxQuantLoader(file=filepath)
    ds = DataSet(
        loader=loader, metadata_path=metadatapath, sample_column="sample"
    )
    
    ds.metadata = ds.metadata[
        [
            "sample",
            "disease",
            "Drug therapy (procedure) (416608005)",
            "Lipid-lowering therapy (134350008)",
        ]
    ]
    ds.preprocess(subset=True)
    st.session_state["loader"] = loader
    st.session_state["metadata_columns"] = ds.metadata.columns.to_list()
    st.session_state["dataset"] = ds

    load_options()


def import_data():

    software = st.selectbox(
        "Select your Proteomics Software",
        options=["<select>", "MaxQuant", "AlphaPept", "DIANN", "FragPipe", "Spectronaut"],
    )

    session_state_empty = False

    if software != "<select>":
        # if
        # reset()
        upload_softwarefile(software=software)

    if "loader" in st.session_state:
        upload_metadatafile(software)


def display_loaded_dataset():

    st.info("Data was successfully imported")
    st.info("DataSet has been created")

    st.markdown(f"*Preview:* Raw data from {st.session_state.dataset.software}")
    st.dataframe(st.session_state.dataset.rawinput.head(5))

    st.markdown(f"*Preview:* Metadata")
    st.dataframe(st.session_state.dataset.metadata.head(5))

    st.markdown(f"*Preview:* Matrix")

    df = pd.DataFrame(
        st.session_state.dataset.mat.values,
        index=st.session_state.dataset.mat.index.to_list(),
    ).head(5)

    st.dataframe(df)


def save_plot_sampledistribution_rawdata():
    df = st.session_state.dataset.rawmat
    df = df.unstack().reset_index()
    df.rename(
        columns={"level_1": st.session_state.dataset.sample, 0: "Intensity"},
        inplace=True,
    )
    st.session_state["distribution_plot"] = px.violin(
        df, x=st.session_state.dataset.sample, y="Intensity"
    )


def empty_session_state():
    """
    remove all variables to avoid conflicts
    """
    for key in st.session_state.keys():
        del st.session_state[key]
    st.empty()

    from streamlit.runtime import get_instance
    from streamlit.runtime.scriptrunner.script_run_context import get_script_run_ctx
    user_session_id = get_script_run_ctx().session_id
    st.session_state["user_session_id"] = user_session_id

sidebar_info()


if "dataset" not in st.session_state:
    st.markdown("### Import Proteomics Data")

    st.markdown(
        "Create a DataSet with the output of your proteomics software package and the corresponding metadata (optional). "
    )

    import_data()
    st.markdown("### Or Load sample Dataset")

if st.button("Load sample DataSet - PXD011839"):

    st.write(
        """

    ### Plasma proteome profiling discovers novel proteins associated with non-alcoholic fatty liver disease

    **Description**

    Non-alcoholic fatty liver disease (NAFLD) affects 25 percent of the population and can progress to cirrhosis, 
    where treatment options are limited. As the liver secrets most of the blood plasma proteins its diseases 
    should affect the plasma proteome. Plasma proteome profiling on 48 patients with cirrhosis or NAFLD with 
    normal glucose tolerance or diabetes, revealed 8 significantly changing (ALDOB, APOM, LGALS3BP, PIGR, VTN, 
    IGHD, FCGBP and AFM), two of which are already linked to liver disease. Polymeric immunoglobulin receptor (PIGR) 
    was significantly elevated in both cohorts with a 2.7-fold expression change in NAFLD and 4-fold change in 
    cirrhosis and was further validated in mouse models. Furthermore, a global correlation map of clinical and 
    proteomic data strongly associated DPP4, ANPEP, TGFBI, PIGR, and APOE to NAFLD and cirrhosis. DPP4 is a known 
    drug target in diabetes. ANPEP and TGFBI are of interest because of their potential role in extracellular matrix 
    remodeling in fibrosis.

    **Publication**

    Niu L, Geyer PE, Wewer Albrechtsen NJ, Gluud LL, Santos A, Doll S, Treit PV, Holst JJ, Knop FK, Vilsbøll T, Junker A, 
    Sachs S, Stemmer K, Müller TD, Tschöp MH, Hofmann SM, Mann M, Plasma proteome profiling discovers novel proteins 
    associated with non-alcoholic fatty liver disease. Mol Syst Biol, 15(3):e8793(2019)
    """
    )

    load_sample_data()

if "dataset" in st.session_state:
    st.info("DataSet has been imported")

    if "distribution_plot" not in st.session_state:
        save_plot_sampledistribution_rawdata()

    if st.button("New Session: Import new dataset"):

        empty_session_state()

        import_data()

    if "dataset" in st.session_state:

        display_loaded_dataset()



import unittest
import pandas as pd
import logging
from unittest.mock import patch
import logging
import shutil
import os
import copy


from alphastats.loader.BaseLoader import BaseLoader
from alphastats.loader.DIANNLoader import DIANNLoader
from alphastats.loader.MaxQuantLoader import MaxQuantLoader
from alphastats.loader.AlphaPeptLoader import AlphaPeptLoader
from alphastats.loader.FragPipeLoader import FragPipeLoader
from alphastats.loader.SpectronautLoader import SpectronautLoader
from alphastats.loader.mzTabLoader import mzTabLoader
from contextlib import contextmanager

logger = logging.getLogger(__name__)


class BaseTestLoader:
    #  parent class of test loader for common tests among loaders
    # this is wrapped in a nested class so it doesnt get called separatly when testing
    # plus to avoid multiple inheritance
    class BaseTest(unittest.TestCase):
        @contextmanager
        def assertNotRaises(self, exc_type):
            try:
                yield None
            except exc_type:
                raise self.failureException("{} raised".format(exc_type.__name__))

        def test_dataformat(self):
            # check if loaded data is pandas dataframe
            self.assertIsInstance(self.obj.rawinput, pd.DataFrame)

        def test_check_if_columns_are_present_error(self):
            # check if columns are present
            # check if error gets raised when column is not present
            with self.assertRaises(KeyError):
                obj = copy.deepcopy(self.obj)
                obj.confidence_column = "wrong_column"
                obj._check_if_columns_are_present()

        def test_check_if_columns_are_present_no_error(self):
            # check if columns are present
            # check if error gets raised when column is not present
            with self.assertNotRaises(KeyError):
                self.obj._check_if_columns_are_present()

        @patch("logging.Logger.warning")
        def test_check_if_indexcolumn_is_unique_warning(self, mock):
            #  check if indexcolumn is unique
            # check if error gets raised when duplicate
            obj = copy.deepcopy(self.obj)
            obj.rawinput[obj.index_column] = "non unique"
            obj._check_if_indexcolumn_is_unique()
            mock.assert_called_once()

        # @patch("logging.Logger.warning")
        # def test_check_if_indexcolumn_is_unique_no_warning(self,mock):
        #  check if indexcolumn is unique
        # self.obj.check_if_indexcolumn_is_unique()
        # mock.assert_not_called()

        def test_check_if_file_exists(self):
            # check if error gets raised when file doesnt exist
            with self.assertRaises(OSError):
                wrong_file_path = "wrong/file/path"
                self.obj._check_if_file_exists(file=wrong_file_path)

        def test_add_contaminantion_column(self):
            column_added = "contamination_library" in self.obj.rawinput
            self.assertTrue(column_added)
            self.assertEqual(self.obj.rawinput["contamination_library"].dtype, "bool")

        def test_df_dimensions(self):
            # test if dataframe gets loaded correctly
            # are there as many rows and column we expect
            self.assertEqual(self.obj.rawinput.shape, self.df_dim)


class TestAlphaPeptLoader(BaseTestLoader.BaseTest):
    @classmethod
    def setUpClass(cls):
        cls.obj = AlphaPeptLoader(file="testfiles/alphapept/results_proteins.csv")
        cls.hdf_file = "testfiles/alphapept/results.hdf"
        # expected dim of rawinput df
        cls.df_dim = (3781, 8)

    def test_load_hdf_protein_table(self):
        hdf_format = AlphaPeptLoader(file=self.hdf_file)
        # test if hdf file gets loaded
        self.assertEqual(hdf_format.rawinput.shape, self.df_dim)

    def test_add_contamination_reverse_column(self):
        # check if contamination column has been added
        column_added = "Reverse" in self.obj.rawinput
        self.assertTrue(column_added)
        # check if column contains True and False
        self.assertEqual(self.obj.rawinput.Reverse.dtype, "bool")

    def test_standardize_protein_group_column(self):
        # check if column ProteinGroup has been added
        column_added = "ProteinGroup" in self.obj.rawinput
        self.assertTrue(column_added)

        # test function with different entries
        entry_one = "sp|P0DMV9|HS71B_HUMAN,sp|P0DMV8|HS71A_HUMAN"
        entry_one_protein_id = self.obj._standardize_protein_group_column(
            entry=entry_one
        )
        self.assertEqual(entry_one_protein_id, "P0DMV9;P0DMV8")

        entry_two = "ENSEMBL:ENSBTAP00000007350"
        entry_two_protein_id = self.obj._standardize_protein_group_column(
            entry=entry_two
        )
        self.assertEqual(entry_two_protein_id, "ENSBTAP00000007350")


class TestMaxQuantLoader(BaseTestLoader.BaseTest):
    @classmethod
    def setUpClass(cls):
        cls.obj = MaxQuantLoader(file="testfiles/maxquant/proteinGroups.txt")
        cls.df_dim = (2611, 2531)

    def test_set_filter_columns_to_true_false(self):
        # check if + has been replaced by TRUE FALSE
        self.assertEqual(self.obj.rawinput["Reverse"].dtype, "bool")
        self.assertEqual(self.obj.rawinput["Only identified by site"].dtype, "bool")
        self.assertEqual(self.obj.rawinput["Potential contaminant"].dtype, "bool")

class TestDIANNLoader(BaseTestLoader.BaseTest):
    @classmethod
    def setUpClass(cls):
        cls.obj = DIANNLoader(file="testfiles/diann/report_final.pg_matrix.tsv")
        cls.df_dim = (10, 26)

    def add_tag_to_sample_columns(self):
        # get number of columns that have tag
        n_taged_samples = len(
            self.obj.rawinput.filter(like="_Intensity").columns.to_list()
        )
        self.assertEqual(n_taged_samples, 20)

    def test_load_protein_data_df(self):
        df = pd.read_csv("testfiles/diann/report_final.pg_matrix.tsv", sep="\t")
        obj = DIANNLoader(df)
        self.assertIsInstance(obj.rawinput, pd.DataFrame)
        self.assertFalse(obj.rawinput.empty)

    def test_remove_filepath_windows(self):
        column_list = [
            "D:\\user\\path\\raw_A.d",
            "D:\\user\\path\\raw_B.d",
            "D:\\user\\path\\raw_C.d",
            "D:\\user\\path\\raw_D.d",
            "D:\\user\\path\\raw_F.d",
        ]
        expected_names = ["raw_A.d", "raw_B.d", "raw_C.d", "raw_D.d", "raw_F.d"]
        data = [[1, 2, 3, 4, 5], [1, 2, 3, 4, 5]]

        df = pd.DataFrame(data, columns=column_list)
        self.obj.rawinput = df
        self.obj._remove_filepath_from_name()
        self.assertEqual(self.obj.rawinput.columns.to_list(), expected_names)

    def test_remove_filepath_unix(self):
        column_list = [
            "path/to/file/raw_A.d",
            "path/to/file/raw_B.d",
            "path/to/file/raw_C.d",
            "path/to/file/raw_D.d",
            "path/to/file/raw_F.d",
        ]
        expected_names = ["raw_A.d", "raw_B.d", "raw_C.d", "raw_D.d", "raw_F.d"]
        data = [[1, 2, 3, 4, 5], [1, 2, 3, 4, 5]]

        df = pd.DataFrame(data, columns=column_list)
        self.obj.rawinput = df
        self.obj._remove_filepath_from_name()
        self.assertEqual(self.obj.rawinput.columns.to_list(), expected_names)


class TestFragPipeLoader(BaseTestLoader.BaseTest):
    @classmethod
    def setUpClass(cls):
        cls.obj = FragPipeLoader(file="testfiles/fragpipe/combined_proteins.tsv")
        cls.df_dim = (10, 37)

class TestmzTabLoader(BaseTestLoader.BaseTest):
    @classmethod
    def setUpClass(cls):
        cls.obj = mzTabLoader(file="testfiles/mzTab/test.mztab")
        cls.df_dim = (283, 265)

class TestSpectronautLoader(BaseTestLoader.BaseTest):
    @classmethod
    def setUpClass(cls):
        
        if os.path.isfile("testfiles/spectronaut/results.tsv") == False:
            shutil.unpack_archive("testfiles/spectronaut/results.tsv.zip", "testfiles/spectronaut/")
        
        cls.obj = SpectronautLoader(file="testfiles/spectronaut/results.tsv", filter_qvalue=False)
        cls.df_dim = (2458, 11)

    def test_reading_non_european_comma(self):
        """
        files with non european comma get read correctly
        """
        s = SpectronautLoader(file="testfiles/spectronaut/results_non_european_comma.tsv", filter_qvalue=False)
        mean = s.rawinput["20221015_EV_TP_40SPD_LITDIA_MS1_Rapid_MS2_Rapid_57w_100ng_03_PG.Quantity"].mean()

    def test_qvalue_filtering(self):
        obj = SpectronautLoader(file="testfiles/spectronaut/results.tsv", filter_qvalue=True, qvalue_cutoff=0.00000001)
        self.assertEqual(obj.rawinput.shape, (2071, 10))

    def test_qvalue_filtering_warning(self):
        with self.assertWarns(Warning):
            df = pd.read_csv("testfiles/spectronaut/results.tsv", sep="\t", decimal=",")
            df.drop(columns = ["EG.Qvalue"], axis=1) 
            SpectronautLoader(file=df)

    def test_gene_name_column(self):
        df = pd.read_csv("testfiles/spectronaut/results.tsv", sep="\t", decimal=",")
        df["PG.Genes"] = 0
        s = SpectronautLoader(file=df, filter_qvalue=False)
  
    @classmethod
    def tearDownClass(cls):
        if os.path.isdir("testfiles/spectronaut/__MACOSX"):
            shutil.rmtree("testfiles/spectronaut/__MACOSX")

        os.remove("testfiles/spectronaut/results.tsv")
       
        


if __name__ == "__main__":
    unittest.main()



from calendar import c
from http.cookiejar import LoadError
from math import remainder

# from multiprocessing.sharedctypes import Value
from random import sample
import unittest
import pandas as pd
import logging
from unittest.mock import patch
import logging
import numpy as np
import pandas as pd
import plotly
from contextlib import contextmanager
import shutil
import os
import copy
import dictdiffer
# from pandas.api.types import is_object_dtype, is_numeric_dtype, is_bool_dtype

from alphastats.loader.BaseLoader import BaseLoader
from alphastats.loader.DIANNLoader import DIANNLoader
from alphastats.loader.MaxQuantLoader import MaxQuantLoader
from alphastats.loader.AlphaPeptLoader import AlphaPeptLoader
from alphastats.loader.FragPipeLoader import FragPipeLoader
from alphastats.loader.SpectronautLoader import SpectronautLoader
from alphastats.DataSet import DataSet

from alphastats.DataSet_Statistics import Statistics
from alphastats.DataSet_Plot import Plot
from alphastats.utils import LoaderError


logger = logging.getLogger(__name__)


class BaseTestDataSet:
    # parent class of test loader for common tests among loaders
    # this is wrapped in a nested class so it doesnt get called separatly when testing
    # plus to avoid multiple inheritance
    class BaseTest(unittest.TestCase):
        @contextmanager
        def assertNotRaises(self, exc_type):
            try:
                yield None
            except exc_type:
                raise self.failureException("{} raised".format(exc_type.__name__))

        def test_check_loader_no_error(self):
            with self.assertNotRaises(ValueError):
                self.obj._check_loader(loader=self.loader)

        def test_check_loader_error_invalid_column(self):
            #  invalid index column
            with self.assertRaises(ValueError):
                self.loader.index_column = 100
                self.obj._check_loader(loader=self.loader)

        def test_check_loader_error_empty_df(self):
            # empty dataframe
            with self.assertRaises(ValueError):
                self.loader.rawinput = pd.DataFrame()
                self.obj._check_loader(loader=self.loader)

        def test_check_loader_error_invalid_loader(self):
            #  invalid loader, class
            with self.assertRaises(LoaderError):
                df = pd.DataFrame()
                self.obj._check_loader(loader=df)

        def test_load_metadata(self):
            #  is dataframe loaded
            self.assertIsInstance(self.obj.metadata, pd.DataFrame)
            self.assertFalse(self.obj.metadata.empty)

        @patch("logging.Logger.error")
        def test_load_metadata_missing_sample_column(self, mock):
            # is error raised when name of sample column is missing
            path = self.metadata_path
            self.obj.sample = "wrong_sample_column"
            self.obj.load_metadata(file_path=path)
            mock.assert_called_once()

        @patch("logging.Logger.warning")
        def test_load_metadata_warning(self, mock):
            # is dataframe None and is warning produced
            file_path = "wrong/file.xxx"
            self.obj.load_metadata(file_path=file_path)
            mock.assert_called_once()

        def test_create_matrix(self):
            #  matrix dimensions
            self.assertEqual(self.obj.mat.shape, self.matrix_dim)
            # does the matrix only contain floats/integers and NAs
            is_dtype_numeric = list(
                set(list(map(pd.api.types.is_numeric_dtype, self.obj.mat.dtypes)))
            )
            self.assertEqual(is_dtype_numeric, [True])

        def test_preprocess_print_info(self):
            self.obj.preprocess_print_info()

        @patch("logging.Logger.warning")
        def test_check_values_warning(self, mock):
            # is dataframe None and is warning produced
            data = {
                "A": [10, 11, 12, 13, 14],
                "B": [23, 22, 24, 22, 25],
                "C": [66, 72, np.inf, 68, -np.inf],
            }
            self.obj.mat = pd.DataFrame(data)
            self.obj._check_matrix_values()
            mock.assert_called_once()

        @patch("logging.Logger.info")
        def test_preprocess_filter(self, mock):
            # is info printed if contamination columns get removed
            # is the new matrix smaller than the older matrix
            self.obj.preprocess(remove_contaminations=True)
            self.assertEqual(self.obj.mat.shape, self.matrix_dim_filtered)
            #  info has been printed at least once
            mock.assert_called_once()

        @patch("logging.Logger.info")
        def test_preprocess_filter_already_filter(self, mock):
            # is warning raised when filter columns are none
            # is info printed if contamination columns get removed
            # is the new matrix smaller than the older matrix
            self.assertFalse(
                self.obj.preprocessing_info.get("Contaminations have been removed")
            )
            self.obj.preprocess(remove_contaminations=True)
            self.assertEqual(self.obj.mat.shape, self.matrix_dim_filtered)
            self.assertTrue(
                self.obj.preprocessing_info.get("Contaminations have been removed")
            )
            self.obj.preprocess(remove_contaminations=True)
            self.assertEqual(self.obj.mat.shape, self.matrix_dim_filtered)

        @patch("logging.Logger.info")
        def test_preprocess_filter_no_filter_columns(self, mock):
            self.obj.filter_columns = []
            self.obj.preprocess(remove_contaminations=True)
            mock.assert_called_once()

        def test_preprocess_normalization_invalid_method(self):
            """
            Raises Error when method is not available for Normalization
            """
            with self.assertRaises(ValueError):
                self.obj.preprocess(normalization="wrong method")

        def test_preprocess_imputation_invalid_method(self):
            with self.assertRaises(ValueError):
                self.obj.preprocess(imputation="wrong method")

        def test_imputation_mean(self):
            self.obj.preprocess(imputation="mean")
            self.assertFalse(self.obj.mat.isna().values.any())

        def test_imputation_median(self):
            self.obj.preprocess(imputation="median")
            self.assertFalse(self.obj.mat.isna().values.any())

        def test_imputation_knn(self):
            self.obj.preprocess(imputation="knn")
            self.assertFalse(self.obj.mat.isna().values.any())

        def test_plot_sampledistribution_wrong_method(self):
            """
            Raises Error when method is not available for plotting Sampledistribution
            """
            with self.assertRaises(ValueError):
                self.obj.plot_sampledistribution(method="wrong_method")

        def test_plot_sampledistribution(self):
            plot = self.obj.plot_sampledistribution(log_scale=True)
            # check if it is a figure
            self.assertIsInstance(plot, plotly.graph_objects.Figure)
            # convert plotly objec to dict
            plot_dict = plot.to_plotly_json()
            # check if plotly object is not empty
            self.assertEqual(len(plot_dict.get("data")), 1)
            #  check if it is logscale
            self.assertEqual(plot_dict.get("layout").get("yaxis").get("type"), "log")
        
        def test_reset_preprocessing(self):
            self.assertEqual(self.obj.mat.shape, self.matrix_dim)
            
            self.obj.preprocess(remove_contaminations=True)
            self.assertEqual(self.obj.mat.shape, self.matrix_dim_filtered)

            self.obj.reset_preprocessing()
            self.assertEqual(self.obj.mat.shape, self.matrix_dim)


class TestAlphaPeptDataSet(BaseTestDataSet.BaseTest):
    #  do testing which requires extra files only on TestAlphaPeptDataSet
    # to reduce the amount of compariosn files required
    def setUp(self):
        self.loader = AlphaPeptLoader(file="testfiles/alphapept/results_proteins.csv")
        self.metadata_path = "testfiles/alphapept/metadata.csv"
        self.obj = DataSet(
            loader=self.loader,
            metadata_path=self.metadata_path,
            sample_column="sample",
        )
        # expected dimensions of matrix
        self.matrix_dim = (2, 3781)
        self.matrix_dim_filtered = (2, 3707)
        #  metadata column to compare for PCA, t-test, etc.
        self.comparison_column = "disease"

    def test_dataset_without_metadata(self):
        obj = DataSet(loader=self.loader)
        self.assertEqual(obj.mat.shape[0], obj.metadata.shape[0])

    def test_load_metadata_fileformats(self):
        # test if different fileformats get loaded correctly
        metadata_path = "testfiles/alphapept/metadata.txt"
        self.obj.load_metadata(file_path=metadata_path)
        self.assertEqual(self.obj.metadata.shape, (2, 2))

        metadata_path = "testfiles/alphapept/metadata.tsv"
        self.obj.load_metadata(file_path=metadata_path)
        self.assertEqual(self.obj.metadata.shape, (2, 2))

        metadata_path = "testfiles/alphapept/metadata.csv"
        self.obj.load_metadata(file_path=metadata_path)
        self.assertEqual(self.obj.metadata.shape, (2, 2))

    @patch("logging.Logger.warning")
    def test_remove_misc_samples_in_metadata(self, mock):
        df = pd.DataFrame(
            {"sample": ["A", "B", "C"], "b": ["disease", "health", "disease"]}
        )
        obj = DataSet(loader=self.loader, metadata_path=df, sample_column="sample",)
        #  is sample C removed
        self.assertEqual(self.obj.metadata.shape, (2, 2))
        mock.assert_called_once()

    def test_load_metadata_df(self):
        if self.metadata_path.endswith(".csv"):
            df = pd.read_csv(self.metadata_path)
        else:
            df = pd.read_excel(self.metadata_path)
        obj = DataSet(loader=self.loader, metadata_path=df, sample_column="sample",)
        self.assertIsInstance(obj.metadata, pd.DataFrame)
        self.assertFalse(obj.metadata.empty)

    def test_preprocess_remove_samples(self):
        sample_list = ["A"]
        self.obj.preprocess(remove_samples=sample_list)
        self.assertEqual(self.obj.mat.shape, (1, 3781))

    def test_preprocess_normalize_zscore(self):
        self.obj.mat = pd.DataFrame({"a": [2, 5, 4], "b": [5, 4, 4], "c": [0, 10, 8]})
        # zscore Normalization
        self.obj.preprocess(log2_transform=False,normalization="zscore")
        expected_mat = pd.DataFrame(
            {
                "a": [-1.33630621, 1.06904497, 0.26726124],
                "b": [1.41421356, -0.70710678, -0.70710678],
                "c": [-1.38873015, 0.9258201, 0.46291005],
            }
        )
        pd._testing.assert_frame_equal(self.obj.mat, expected_mat)

    def test_preprocess_normalize_quantile(self):
        self.obj.mat = pd.DataFrame({"a": [2, 5, 4], "b": [5, 4, 4], "c": [0, 10, 8]})
        # Quantile Normalization
        self.obj.preprocess(log2_transform=False,normalization="quantile")
        expected_mat = pd.DataFrame(
            {"a": [0.0, 1.0, 0.5], "b": [1.0, 0.0, 0.0], "c": [0.0, 1.0, 0.5]}
        )
        pd._testing.assert_frame_equal(self.obj.mat, expected_mat)

    def test_preprocess_normalize_linear(self):
        self.obj.mat = pd.DataFrame({"a": [2, 5, 4], "b": [5, 4, 4], "c": [0, 10, 8]})
        # Linear Normalization
        self.obj.preprocess(log2_transform=False,normalization="linear")
        expected_mat = pd.DataFrame(
            {
                "a": [0.37139068, 0.42107596, 0.40824829],
                "b": [0.92847669, 0.33686077, 0.40824829],
                "c": [0.0, 0.84215192, 0.81649658],
            }
        )
        pd._testing.assert_frame_equal(self.obj.mat, expected_mat)

    def test_preprocess_normalize_vst(self):
        self.obj.mat = pd.DataFrame({"a": [2, 5, 4], "b": [5, 4, 4], "c": [0, 10, 8]})
        # Linear Normalization
        self.obj.preprocess(log2_transform=False,normalization="vst")
        expected_mat = pd.DataFrame(
            {
                "a": [-1.30773413, 1.12010046, 0.18763367],
                "b": [1.41421361, -0.70710674, -0.70710674],
                "c": [-1.39384919, 0.90401955, 0.48982964],
            }
        )
        pd._testing.assert_frame_equal(self.obj.mat, expected_mat)

    def test_preprocess_imputation_mean_values(self):
        self.obj.mat = pd.DataFrame(
            {"a": [2, np.nan, 4], "b": [5, 4, 4], "c": [np.nan, 10, np.nan]}
        )
        self.obj.preprocess(log2_transform=False,imputation="mean")
        expected_mat = pd.DataFrame(
            {"a": [2.0, 3.0, 4.0], "b": [5.0, 4.0, 4.0], "c": [10.0, 10.0, 10.0]}
        )
        pd._testing.assert_frame_equal(self.obj.mat, expected_mat)

    def test_preprocess_imputation_median_values(self):
        self.obj.mat = pd.DataFrame(
            {"a": [2, np.nan, 4], "b": [5, 4, 4], "c": [np.nan, 10, np.nan]}
        )
        self.obj.preprocess(log2_transform=False,imputation="median")
        expected_mat = pd.DataFrame(
            {"a": [2.0, 3.0, 4.0], "b": [5.0, 4.0, 4.0], "c": [10.0, 10.0, 10.0]}
        )
        pd._testing.assert_frame_equal(self.obj.mat, expected_mat)

    def test_preprocess_imputation_knn_values(self):
        self.obj.mat = pd.DataFrame(
            {"a": [2, np.nan, 4], "b": [5, 4, 4], "c": [np.nan, 10, np.nan]}
        )
        self.obj.preprocess(log2_transform=False,imputation="knn")
        expected_mat = pd.DataFrame(
            {"a": [2.0, 3.0, 4.0], "b": [5.0, 4.0, 4.0], "c": [10.0, 10.0, 10.0]}
        )
        pd._testing.assert_frame_equal(self.obj.mat, expected_mat)

    def test_preprocess_imputation_randomforest_values(self):
        self.obj.mat = pd.DataFrame(
            {"a": [2, np.nan, 4], "b": [5, 4, 4], "c": [np.nan, 10, np.nan]}
        )
        self.obj.preprocess(log2_transform=False,imputation="randomforest")
        expected_mat = pd.DataFrame(
            {
                "a": [2.00000000e00, -9.22337204e12, 4.00000000e00],
                "b": [5.00000000e00, 4.00000000e00, 4.0],
                "c": [-9.22337204e12, 1.00000000e01, -9.22337204e12],
            }
        )
        pd._testing.assert_frame_equal(self.obj.mat, expected_mat)

    def test_plot_sampledistribution_group(self):
        plot = self.obj.plot_sampledistribution(
            method="box", color="disease", log_scale=False
        )
        # check if it is a figure
        self.assertIsInstance(plot, plotly.graph_objects.Figure)
        # convert plotly object to dict
        plot_dict = plot.to_plotly_json()
        #  check if it doesnt get transformed to logscale
        self.assertEqual(plot_dict.get("layout").get("yaxis").get("type"), None)
        # check if there are two groups control and disease
        self.assertEqual(plot_dict.get("data")[0].get("legendgroup"), "control")
        #  check that it is boxplot and not violinplot
        is_boxplot = "boxmode" in plot_dict.get("layout").keys()
        self.assertTrue(is_boxplot)

    def test_plot_correlation_matrix(self):
        plot = self.obj.plot_correlation_matrix()
        plot_dict = plot.to_plotly_json()
        correlation_calculations_expected = [1.0, 0.999410773629427]
        self.assertEqual(
            plot_dict.get("data")[0].get("z")[0].tolist(),
            correlation_calculations_expected,
        )

    def test_plot_clustermap(self):
        self.obj.preprocess(log2_transform=False, imputation="knn")
        plot = self.obj.plot_clustermap()
        first_row = plot.data2d.iloc[0].to_list()
        expected = [487618.5371077078, 1293013.103298046]
        self.assertEqual(first_row, expected)

    def test_plot_clustermap_with_label_bar(self):
        self.obj.preprocess(log2_transform=False, imputation="knn")
        plot = self.obj.plot_clustermap(label_bar=self.comparison_column)
        first_row = plot.data2d.iloc[0].to_list()
        expected = [487618.5371077078, 1293013.103298046]
        self.assertEqual(first_row, expected)


class TestMaxQuantDataSet(BaseTestDataSet.BaseTest):
    def setUp(self):
        self.loader = MaxQuantLoader(file="testfiles/maxquant/proteinGroups.txt")
        self.metadata_path = "testfiles/maxquant/metadata.xlsx"
        self.obj = DataSet(
            loader=self.loader,
            metadata_path=self.metadata_path,
            sample_column="sample",
        )
        # expected dimensions of matrix
        self.matrix_dim = (312, 2596)
        self.matrix_dim_filtered = (312, 2397)
        self.comparison_column = "disease"

    def test_load_evidence_wrong_sample_names(self):
        with self.assertRaises(ValueError):
            loader = MaxQuantLoader(
                file="testfiles/maxquant/proteinGroups.txt",
                evidence_file="testfiles/maxquant_go/evidence.txt"
            )
            DataSet(
                loader=loader,
                metadata_path=self.metadata_path,
                sample_column="sample",
            )

    def test_plot_pca_group(self):
        pca_plot = self.obj.plot_pca(group=self.comparison_column)
        # 5 different disease
        self.assertEqual(len(pca_plot.to_plotly_json().get("data")), 5)

    def test_plot_pca_circles(self):
        pca_plot = self.obj.plot_pca(group=self.comparison_column, circle=True)
        # are there 5 circles drawn - for each group
        number_of_groups = len(pca_plot.to_plotly_json().get("layout").get("shapes"))
        self.assertEqual(number_of_groups, 5)

    def test_plot_umap_group(self):
        umap_plot = self.obj.plot_umap(group=self.comparison_column)
        # 5 different disease
        self.assertEqual(len(umap_plot.to_plotly_json().get("data")), 5)

    def test_plot_umap_circles(self):
        umap_plot = self.obj.plot_umap(group=self.comparison_column, circle=True)
        # are there 5 circles drawn - for each group
        number_of_groups = len(umap_plot.to_plotly_json().get("layout").get("shapes"))
        self.assertEqual(number_of_groups, 5)

    def test_plot_volcano_with_grouplist(self):
        fig = self.obj.plot_volcano(
            method="ttest",
            group1=["1_31_C6", "1_32_C7", "1_57_E8"],
            group2=["1_71_F10", "1_73_F12"],
        )

    def test_plot_volcano_with_grouplist_wrong_names(self):
        with self.assertRaises(ValueError):
            self.obj.plot_volcano(
                method="ttest",
                group1=["wrong_sample_name", "1_42_D9", "1_57_E8"],
                group2=["1_71_F10", "1_73_F12"],
            )
    
    def test_plot_volcano_compare_preprocessing_modes(self):
        result_list = self.obj.plot_volcano(
            method="ttest",
            group1=["1_31_C6", "1_32_C7", "1_57_E8"],
            group2=["1_71_F10", "1_73_F12"],
            compare_preprocessing_modes=True
        )
        self.assertEqual(len(result_list), 9)               

    def test_preprocess_subset(self):
        self.obj.preprocess(subset=True, log2_transform=False)
        self.assertEqual(self.obj.mat.shape, (48, 1364))

    @patch.object(Statistics, "tukey_test")
    def test_anova_without_tukey(self, mock):
        anova_results = self.obj.anova(column="disease", protein_ids="all", tukey=False)
        self.assertEqual(anova_results["ANOVA_pvalue"][1], 0.4469688936240973)
        self.assertEqual(anova_results.shape, (2600, 2))
        # check if tukey isnt called
        mock.assert_not_called()

    def test_plot_intenstity_subgroup(self):
        plot = self.obj.plot_intensity(
            protein_id="K7ERI9;A0A024R0T8;P02654;K7EJI9;K7ELM9;K7EPF9;K7EKP1",
            group="disease",
            subgroups=["healthy", "liver cirrhosis"],
            add_significance=True,
        )
        plot_dict = plot.to_plotly_json()
        self.assertEqual(len(plot_dict.get("data")), 3)

    @patch("logging.Logger.warning")
    def test_plot_intenstity_subgroup_significance_warning(self, mock):
        plot = self.obj.plot_intensity(
            protein_id="K7ERI9;A0A024R0T8;P02654;K7EJI9;K7ELM9;K7EPF9;K7EKP1",
            group="disease",
            add_significance=True,
        )
        plot_dict = plot.to_plotly_json()
        self.assertEqual(len(plot_dict.get("data")), 5)
        mock.assert_called_once()

    def test_anova_with_tukey(self):
        # with first 100 protein ids
        self.obj.preprocess(imputation="mean")
        id_list = self.obj.mat.columns.tolist()[0:100]
        results = self.obj.anova(column="disease", protein_ids=id_list, tukey=True)
        self.assertEqual(results.shape, (100, 10))

        # with one protein id
        protein_id = "A0A024R4J8;Q92876"
        results = self.obj.anova(column="disease", protein_ids=protein_id, tukey=True)
        self.assertEqual(results.shape[1], 10)

    def test_tukey_test(self):
        protein_id = "K7ERI9;A0A024R0T8;P02654;K7EJI9;K7ELM9;K7EPF9;K7EKP1"
        tukey_df = self.obj.tukey_test(
            protein_id=protein_id, group="disease", df=None
        )
        self.assertEqual(tukey_df["p-tukey"][0], 0.674989009816342)

    def test_ancova(self):
        ancova_df = self.obj.ancova(
            protein_id="K7ERI9;A0A024R0T8;P02654;K7EJI9;K7ELM9;K7EPF9;K7EKP1",
            covar="Triglycerides measurement (14740000)",
            between="disease",
        )
        expected_value = 0.7375624497867097
        given_value = ancova_df["p-unc"][1]
        decimal_places = 7
        self.assertAlmostEqual(expected_value, given_value, decimal_places)

    def test_plot_volcano_with_labels(self):
        plot = self.obj.plot_volcano(
            column="disease",
            group1="healthy",
            group2="liver cirrhosis",
            method="ttest",
            labels=True,
            draw_line=False,
        )
        n_labels = len(plot.to_plotly_json().get("layout").get("annotations"))
        #self.assertTrue(n_labels > 20)

    def test_plot_volcano_wald(self):
        """
        Volcano Plot with wald test and list of samples
        """
        self.obj.preprocess(imputation="knn")
        self.obj.plot_volcano(group1 = ["1_31_C6", "1_32_C7", "1_33_C8"],
                    group2 = ["1_78_G5", "1_77_G4", "1_76_G3"], method="ttest")

        column_added = "_comparison_column" in self.obj.metadata.columns.to_list()
        self.assertTrue(column_added)  

    def test_plot_volcano_sam(self):
        self.obj.preprocess(log2_transform=False, imputation="knn", normalization="zscore")
        plot = self.obj.plot_volcano(
            column = "disease",
            group1="type 2 diabetes mellitus", 
            group2 ="type 2 diabetes mellitus|non-alcoholic fatty liver disease",
            method="sam",
            draw_line =True,
            perm= 10
        )

        # fdr lines get drawn
        line_1 = plot.to_plotly_json()["data"][3].get("line").get("shape")
        line_2 = plot.to_plotly_json()["data"][4].get("line").get("shape")

        self.assertEqual(line_1, "spline")
        self.assertEqual(line_2, "spline")
    
    def test_plot_volcano_list(self):
        self.obj.preprocess(imputation="mean")
        plot = self.obj.plot_volcano( method="ttest",
            group1=["1_31_C6", "1_32_C7", "1_57_E8"],
            group2=["1_71_F10", "1_73_F12"],
            color_list=self.obj.mat.columns.to_list()[0:20])
        self.assertEqual(len(plot.to_plotly_json()["data"][0]["x"]), 20)


    def test_plot_clustermap_significant(self):
        import sys
        sys.setrecursionlimit(100000)
        self.obj.preprocess(imputation="knn")
        plot = self.obj.plot_clustermap(
            label_bar=self.comparison_column,
            only_significant=True,
            group=self.comparison_column,
            subgroups=["healthy", "liver cirrhosis"],
        )

    def test_plot_volcano_with_labels_proteins(self):
        # remove gene names
        self.obj.gene_names = None
        plot = self.obj.plot_volcano(
            column="disease",
            group1="healthy",
            group2="liver cirrhosis",
            method="ttest",
            labels=True,
        )
        n_labels = len(plot.to_plotly_json().get("layout").get("annotations"))
    
    def test_plot_volcano_with_labels_proteins_welch_ttest(self):
        # remove gene names
        self.obj.gene_names = None
        plot = self.obj.plot_volcano(
            column="disease",
            group1="healthy",
            group2="liver cirrhosis",
            method="welch-ttest",
            labels=True,
        )
        n_labels = len(plot.to_plotly_json().get("layout").get("annotations"))
        #self.assertTrue(n_labels > 20)

    def test_calculate_diff_exp_wrong(self):
        # get groups from comparison column
        with self.assertRaises(ValueError):
            self.obj.preprocess(imputation="knn")
            groups = list(set(self.obj.metadata[self.comparison_column].to_list()))
            group1, group2 = groups[0], groups[1]

            self.obj.diff_expression_analysis(
                column=self.comparison_column,
                group1=group1,
                group2=group2,
                method="wrong_method",
            )  # check if dataframe gets created

    def test_diff_expression_analysis_nocolumn(self):
        with self.assertRaises(ValueError):
            self.obj.diff_expression_analysis(
                group1="healthy", group2="liver cirrhosis"
            )

    def test_diff_expression_analysis_list(self):
        self.obj.diff_expression_analysis(
            group1 = ["1_31_C6", "1_32_C7", "1_33_C8"],
            group2 = ["1_78_G5", "1_77_G4", "1_76_G3"], method="ttest")

        column_added = "_comparison_column" in self.obj.metadata.columns.to_list()
        self.assertTrue(column_added)  

    def test_plot_intensity_non_sign(self):
        """
        No significant label is added to intensity plot
        """
        plot = self.obj.plot_intensity(protein_id="S6BAR0", 
            group="disease", 
            subgroups=["liver cirrhosis", "healthy"],
            add_significance=True)

        annotation = plot.to_plotly_json().get("layout").get("annotations")[1].get("text")
        self.assertEqual(annotation, "-")

    def test_plot_intensity_sign(self):
        """
        Significant label * is added to intensity plot
        """
        plot = self.obj.plot_intensity(protein_id="Q9UL94", 
            group="disease", 
            subgroups=["liver cirrhosis", "healthy"],
            add_significance=True)

        annotation = plot.to_plotly_json().get("layout").get("annotations")[1].get("text")
        self.assertEqual(annotation, "*")

    def test_plot_intensity_sign_01(self):
        """
        Significant label ** is added to intensity plot
        """
        plot = self.obj.plot_intensity(protein_id="Q96JD0;Q96JD1;P01721", 
            group="disease", 
            subgroups=["liver cirrhosis", "healthy"],
            add_significance=True)

        annotation = plot.to_plotly_json().get("layout").get("annotations")[1].get("text")
        self.assertEqual(annotation, "**")

    def test_plot_intensity_sign_001(self):
        """
        Highly significant label is added to intensity plot
        """
        plot = self.obj.plot_intensity(protein_id="Q9BWP8", 
            group="disease", 
            subgroups=["liver cirrhosis", "healthy"],
            add_significance=True)

        annotation = plot.to_plotly_json().get("layout").get("annotations")[1].get("text")
        self.assertEqual(annotation, "***")

    def test_plot_intensity_all(self):
        plot = self.obj.plot_intensity(protein_id="Q9BWP8", 
            group="disease", 
            subgroups=["liver cirrhosis", "healthy"],
            method="all",
            add_significance=True)
        self.assertEqual(plot.to_plotly_json()["data"][0]["points"], "all")

    
    def test_plot_samplehistograms(self):
        fig = self.obj.plot_samplehistograms().to_plotly_json()
        self.assertEqual(312, len(fig["data"]))
    
    def test_batch_correction(self):
        self.obj.preprocess(subset=True, imputation="knn", normalization="quantile")
        self.obj.batch_correction(batch="batch_artifical_added")
        first_value = self.obj.mat.values[0,0]
        self.assertAlmostEqual(0.0111, first_value, places=2)


        

    # def test_perform_gsea(self):
    #     df = self.obj.perform_gsea(column="disease", 
    #                             group1="healthy", 
    #                                     group2="liver cirrhosis",
    #                                     gene_sets= 'KEGG_2019_Human')

    #     cholesterol_enhanced = 'Cholesterol metabolism' in df.index.to_list()
    #     self.assertTrue(cholersterol_enhanced)


class TestDIANNDataSet(BaseTestDataSet.BaseTest):
    def setUp(self):
        self.loader = DIANNLoader(file="testfiles/diann/report_final.pg_matrix.tsv")
        self.metadata_path = "testfiles/diann/metadata.xlsx"
        self.obj = DataSet(
            loader=self.loader,
            metadata_path=self.metadata_path,
            sample_column="analytical_sample external_id",
        )
        # expected dimensions of matrix
        self.matrix_dim = (20, 10)
        self.matrix_dim_filtered = (20, 10)
        self.comparison_column = "grouping1"

    def test_plot_intensity_violin(self):
        # Violinplot
        plot = self.obj.plot_intensity(
            protein_id="A0A075B6H7", group="grouping1", method="violin"
        )
        plot_dict = plot.to_plotly_json()
        self.assertIsInstance(plot, plotly.graph_objects.Figure)
        # are two groups plotted
        self.assertEqual(len(plot_dict.get("data")), 2)

    def test_plot_intensity_box(self):
        # Boxplot
        plot = self.obj.plot_intensity(
            protein_id="A0A075B6H7", group="grouping1", method="box", log_scale=True
        )
        plot_dict = plot.to_plotly_json()
        #  log scale
        self.assertEqual(plot_dict.get("layout").get("yaxis").get("type"), "log")
        is_boxplot = "boxmode" in plot_dict.get("layout").keys()
        self.assertTrue(is_boxplot)

    def test_plot_intensity_scatter(self):
        # Scatterplot
        plot = self.obj.plot_intensity(
            protein_id="A0A075B6H7", group="grouping1", method="scatter"
        )
        plot_dict = plot.to_plotly_json()
        self.assertIsInstance(plot, plotly.graph_objects.Figure)
        # are two groups plotted
        self.assertEqual(plot_dict.get("data")[0].get("type"), "scatter")

    def test_plot_intensity_wrong_method(self):
        with self.assertRaises(ValueError):
            self.obj.plot_intensity(
                protein_id="A0A075B6H7", group="grouping1", method="wrong"
            )

    def test_plot_clustermap_noimputation(self):
        # raises error when data is not imputed
        with self.assertRaises(ValueError):
            self.obj.plot_clustermap()

    def test_plot_dendrogram(self):
        self.obj.preprocess(imputation="mean")
        fig = self.obj.plot_dendrogram()

    def test_plot_tsne(self):
        plot_dict = self.obj.plot_tsne().to_plotly_json()
        # check if everything get plotted
        self.assertEqual(len(plot_dict.get("data")[0].get("x")), 20)

    def test_plot_dendrogram_navalues(self):
        with self.assertRaises(ValueError):
            self.obj.plot_dendrogram()

    def test_plot_dendrogram_not_imputed(self):
        with self.assertRaises(ValueError):
            self.obj.plot_dendrogram()

    def test_volcano_plot_anova(self):
        self.obj.preprocess(imputation="knn")
        plot = self.obj.plot_volcano(
            column="grouping1", group1="Healthy", group2="Disease", method="anova"
        )
        expected_y_value = 0.040890177695653236
        y_value = plot.to_plotly_json().get("data")[0].get("y")[1]
        self.assertAlmostEqual(y_value, expected_y_value)

    def test_volcano_plot_ttest_no_column(self):
        with self.assertRaises(ValueError):
            self.obj.preprocess(imputation="knn")
            self.obj.plot_volcano(group1="Healthy", group2="Disease", method="ttest")

    def test_volcano_plot_wrongmethod(self):
        with self.assertRaises(ValueError):
            self.obj.plot_volcano(
                column="grouping1",
                group1="Healthy",
                group2="Disease",
                method="wrongmethod",
            )

    # def test_diff_expression_analysis_with_list(self):
    #     self.obj.preprocess(imputation="knn")
    #     column="grouping1"
    #     group1="Healthy"
    #     group2="Disease"
    #     group1_samples = self.obj.metadata[self.obj.metadata[column] == group1][
    #             "sample"
    #         ].tolist()
    #     group2_samples = self.obj.metadata[self.obj.metadata[column] == group2][
    #             "sample"
    #         ].tolist()
    #     self.obj.diff_expression_analysis(
    #         group1=group1_samples,
    #         group2=group2_samples)


class TestFragPipeDataSet(BaseTestDataSet.BaseTest):
    def setUp(self):
        self.loader = FragPipeLoader(
            file="testfiles/fragpipe/combined_proteins.tsv",
            intensity_column="[sample] Razor Intensity",
        )
        self.metadata_path = "testfiles/fragpipe/metadata.xlsx"
        self.obj = DataSet(
            loader=self.loader,
            metadata_path=self.metadata_path,
            sample_column="analytical_sample external_id",
        )
        # expected dimensions of matrix
        self.matrix_dim = (20, 6)
        self.matrix_dim_filtered = (20, 6)
        self.comparison_column = "grouping1"


class TestSpectronautDataSet(BaseTestDataSet.BaseTest):
    @classmethod
    def setUpClass(cls):
        
        if os.path.isfile("testfiles/spectronaut/results.tsv") == False:
            shutil.unpack_archive("testfiles/spectronaut/results.tsv.zip", "testfiles/spectronaut/")
        
        cls.cls_loader = SpectronautLoader(file="testfiles/spectronaut/results.tsv", filter_qvalue=False)
        cls.cls_metadata_path = "testfiles/spectronaut/metadata.xlsx"
        cls.cls_obj = DataSet(
            loader=cls.cls_loader,
            metadata_path=cls.cls_metadata_path,
            sample_column="sample"
        )

    def setUp(self):
        self.loader = copy.deepcopy(self.cls_loader)
        self.metadata_path = copy.deepcopy(self.cls_metadata_path)
        self.obj = copy.deepcopy(self.cls_obj)
        self.matrix_dim = (9,2458)
        self.matrix_dim_filtered = (9, 2453)
        self.comparison_column = "condition"
    
    @classmethod
    def tearDownClass(cls):
        if os.path.isdir("testfiles/spectronaut/__MACOSX"):
            shutil.rmtree("testfiles/spectronaut/__MACOSX")

        os.remove("testfiles/spectronaut/results.tsv")



if __name__ == "__main__":
    unittest.main()



import pandas as pd
import numpy as np

import scipy
from typing import Union

class DifferentialExpressionAnalysis:
    def __init__(
            self,
            dataset,
            group1: Union[str, list],
            group2: Union[str, list],
            column: str = None,
            method: str = "ttest",
            perm: int = 10,
            fdr: float = 0.05,
        ):

        self.dataset = dataset
        self.group1 = group1
        self.group2 = group2
        self.column = column
        self.method = method
        self.perm = perm
        self.fdr = fdr
    
    def _check_groups(self):
        if isinstance(self.group1, list) and isinstance(self.group2, list):
            self.column, self.group1, self.group2 = self._add_metadata_column(self.group1, self.group2)

        elif self.column is None:
            raise ValueError(
                "Column containing group1 and group2 needs to be specified"
            )

    def _prepare_anndata(self):
        import anndata
        group_samples = self.dataset.metadata[
            (self.dataset.metadata[self.column] == self.group1) | (self.dataset.metadata[self.column] == self.group2)
        ][self.dataset.sample].tolist()

        # reduce matrix
        reduced_matrix = self.dataset.mat.loc[group_samples]
        reduced_matrix = reduced_matrix.loc[:, (reduced_matrix != 0).any(axis=0)]
        # sort metadata according to matrix values
        list_to_sort = reduced_matrix.index.to_list()
        #  reduce metadata
        obs_metadata = (
            self.dataset.metadata[self.dataset.metadata[self.dataset.sample].isin(group_samples)]
            .set_index(self.dataset.sample)
            .loc[list_to_sort]
        )

        # change comparison group to 0/1
        obs_metadata[self.column] = np.where(obs_metadata[self.column] == self.group1, 1, 0)

        # create a annotated dataset
        anndata_data = anndata.AnnData(
            X=reduced_matrix.values,
            var=pd.DataFrame(index=reduced_matrix.columns.to_list()),
            obs=obs_metadata,
            dtype=reduced_matrix.values.dtype,
        )
        return anndata_data

    def _add_metadata_column(self, group1_list:list, group2_list:list):
        # create new column in metadata with defined groups
        metadata = self.dataset.metadata

        sample_names = metadata[self.dataset.sample].to_list()

        misc_samples = list(set(group1_list + group2_list) - set(sample_names))
        if len(misc_samples) > 0:
            raise ValueError(
                f"Sample names: {misc_samples} are not described in Metadata."
            )

        column = "_comparison_column"
        conditons = [
            metadata[self.dataset.sample].isin(group1_list),
            metadata[self.dataset.sample].isin(group2_list),
        ]
        choices = ["group1", "group2"]
        metadata[column] = np.select(conditons, choices, default=np.nan)
        self.dataset.metadata = metadata

        return column, "group1", "group2"

    def sam(self) -> pd.DataFrame:
        from alphastats.multicova import multicova
        transposed = self.dataset.mat.transpose()

        if self.dataset.preprocessing_info["Normalization"] is None:
            # needs to be lpog2 transformed for fold change calculations
            transposed = transposed.transform(lambda x: np.log2(x))

        transposed[self.dataset.index_column] = transposed.index
        transposed = transposed.reset_index(drop=True)

        res, _ = multicova.perform_ttest_analysis(
            transposed,
            c1 =list(self.dataset.metadata[self.dataset.metadata[self.column]==self.group1][self.dataset.sample]),                                      
            c2 =list(self.dataset.metadata[self.dataset.metadata[self.column]==self.group2][self.dataset.sample]), 
            s0=0.05, 
            n_perm=self.perm,
            fdr=self.fdr,
            id_col=self.dataset.index_column,
            parallelize=True
        )

        fdr_column = "FDR"  + str(int(self.fdr*100)) + "%"
        df = res[[self.dataset.index_column, 'fc', 'tval', 'pval', 'tval_s0', 'pval_s0', 'qval']]
        df["log2fc"] = res["fc"]
        df["FDR"] = res[fdr_column]

        return df      

    def wald(self) -> pd.DataFrame:
        import diffxpy.api as de
        d = self._prepare_anndata()
        formula_loc = "~ 1 +" + self.column
        
        test = de.test.wald(
            data=d, formula_loc=formula_loc, factor_loc_totest=self.column
        )
        df = test.summary().rename(columns={"gene": self.dataset.index_column})
        return df
    
    def welch_ttest(self) -> pd.DataFrame:
        import diffxpy.api as de
        d = self._prepare_anndata()
        
        test = de.test.t_test(data=d, grouping=self.column)
        df = test.summary().rename(columns={"gene": self.dataset.index_column})
        return df

    def ttest(self) -> pd.DataFrame:
        group1_samples = self.dataset.metadata[self.dataset.metadata[self.column] == self.group1][
            self.dataset.sample
        ].tolist()
        group2_samples = self.dataset.metadata[self.dataset.metadata[self.column] == self.group2][
            self.dataset.sample
        ].tolist()
        # calculate fold change (if its is not logarithimic normalized)
        mat_transpose = self.dataset.mat.transpose()
        
        p_values = mat_transpose.apply(
            lambda row: scipy.stats.ttest_ind(
                row[group1_samples].values.flatten(),
                row[group2_samples].values.flatten(),
            )[1],
            axis=1,
        )

        fc = self._calculate_foldchange(
            mat_transpose=mat_transpose, 
            group1_samples=group1_samples, 
            group2_samples=group2_samples
        )
        df = pd.DataFrame()
        df[self.dataset.index_column], df["pval"] = p_values.index.tolist(), p_values.values
        df["log2fc"] = fc  
        return df

    def pairedttest(self) -> pd.DataFrame:
        group1_samples = self.dataset.metadata[self.dataset.metadata[self.column] == self.group1][
            self.dataset.sample
        ].tolist()
        group2_samples = self.dataset.metadata[self.dataset.metadata[self.column] == self.group2][
            self.dataset.sample
        ].tolist()
        # calculate fold change (if its is not logarithimic normalized)
        mat_transpose = self.dataset.mat.transpose()
        
        p_values = mat_transpose.apply(
            lambda row: scipy.stats.ttest_rel(
                row[group1_samples].values.flatten(),
                row[group2_samples].values.flatten(),
            )[1],
            axis=1,
        )

        fc = self._calculate_foldchange(
            mat_transpose=mat_transpose, 
            group1_samples=group1_samples, 
            group2_samples=group2_samples
        )
        df = pd.DataFrame()
        df[self.dataset.index_column], df["pval"] = p_values.index.tolist(), p_values.values
        df["log2fc"] = fc
        return df

    def _calculate_foldchange(self, mat_transpose:pd.DataFrame, group1_samples:list, group2_samples:list):
        mat_transpose += 0.00001

        if self.dataset.preprocessing_info["Log2-transformed"]:
            fc = (
                mat_transpose[group1_samples].T.mean().values
                - mat_transpose[group2_samples].T.mean().values
            )
        
        else:
            fc = (
                mat_transpose[group1_samples].T.mean().values
                / mat_transpose[group2_samples].T.mean().values
            )
            fc = np.log2(fc)

        return fc

    def perform(self) -> pd.DataFrame:
        self._check_groups()

        if self.method == "wald":
            df = self.wald()
        
        elif self.method == "ttest":
            df = self.ttest()
        
        elif self.method == "welch-ttest":
            df = self.welch_ttest()
        
        elif self.method == "sam":
            df = self.sam()
        
        elif self.method == "paired-ttest":
            df = self.pairedttest()
        
        else:
            raise ValueError(
                f"{self.method} is invalid choose between 'wald' for Wald-test, 'sam',  and 'ttest', 'welch-ttest' or 'paired-ttest'"
            )
        
        return df







# builtin
import setuptools
import re
import os

# local
# info has to be imported individually or sidepacakges will already be installed
# from alphastats import __project__, __version__, __license__, __description__,__author__,__author_email__
# from alphastats import __github__, __keywords__, __python_version__, __classifiers__, __urls__, __extra_requirements__, __console_scripts__


def get_long_description():
    with open("README.md", "r") as readme_file:
        long_description = readme_file.read()
    return long_description


def get_requirements():
    with open('requirements.txt') as f:
        required = f.read().splitlines()
    return required
    

def create_pip_wheel():
    requirements = get_requirements()
    setuptools.setup(
        name="alphastats",
        version="0.6.0",
        license="Apache",
        description="An open-source Python package for Mass Spectrometry Analysis",
        long_description=get_long_description(),
        long_description_content_type="text/markdown",
        author="Mann Labs",
        author_email="elena.krismer@hotmail.com",
        url="https://github.com/MannLabs/alphastats",
        project_urls={
            "Mann Labs at MPIB": "https://www.biochem.mpg.de/mann",
            "GitHub": "https://github.com/MannLabs/alphapeptstats",
            "ReadTheDocs": "https://mannlabs.github.io/alphapeptstats/",
            "PyPi": "https://pypi.org/project/alphastats/"
            # "Scientific paper": None,
        },
        keywords=["bioinformatics", "software", "mass spectometry",],
        classifiers=[
            "Development Status :: 1 - Planning",
            # "Development Status :: 2 - Pre-Alpha",
            # "Development Status :: 3 - Alpha",
            # "Development Status :: 4 - Beta",
            # "Development Status :: 5 - Production/Stable",
            # "Development Status :: 6 - Mature",
            # "Development Status :: 7 - Inactive"
            "Intended Audience :: Science/Research",
            "License :: OSI Approved :: Apache Software License",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3",
            "Topic :: Scientific/Engineering :: Bio-Informatics",
        ],
        packages=[
            "alphastats", 
            "alphastats.plots", 
            "alphastats.multicova",
            "alphastats.gui", 
            "alphastats.data", 
            "alphastats.gui.pages",
            "alphastats.gui",
            "alphastats.gui.sample_data",
            "alphastats.gui.utils",
            "alphastats.loader"
        ],
        include_package_data=True,
        entry_points={"console_scripts": "alphastats=alphastats.gui.gui:run",},
        install_requires=requirements,
        python_requires=">=3.8,<4",
    )


if __name__ == "__main__":
    create_pip_wheel()



import sklearn
import logging
import plotly.express as px
import plotly
import scipy
import sklearn.manifold

import plotly.graph_objects as go
import numpy as np
import seaborn as sns
import pandas as pd
from scipy.spatial.distance import pdist, squareform
import random
import itertools
import plotly.figure_factory

from alphastats.plots.DimensionalityReduction import DimensionalityReduction
from alphastats.plots.VolcanoPlot import VolcanoPlot
from alphastats.plots.IntensityPlot import IntensityPlot
from alphastats.plots.ClusterMap import ClusterMap
from alphastats.plots.SampleHistogram import SampleHistogram
from alphastats.utils import ignore_warning, check_for_missing_values


class plotly_object(plotly.graph_objs._figure.Figure):
    plotting_data = None
    preprocessing = None
    method = None


class seaborn_object(sns.matrix.ClusterGrid):
    plotting_data = None
    preprocessing = None
    method = None


plotly.io.templates["alphastats_colors"] = plotly.graph_objects.layout.Template(
    layout=plotly.graph_objects.Layout(
        paper_bgcolor="rgba(0,0,0,0)",
        plot_bgcolor="rgba(0,0,0,0)",
        colorway=[
            "#009599",
            "#005358",
            "#772173",
            "#B65EAF",  # pink
            "#A73A00",
            "#6490C1",
            "#FF894F",
            "#2B5E8B",
            "#A87F32",
        ],
    )
)

plotly.io.templates.default = "simple_white+alphastats_colors"

class Plot:
    def _update_figure_attributes(self, figure_object, plotting_data, method=None):
        setattr(figure_object, "plotting_data", plotting_data)
        setattr(figure_object, "preprocessing", self.preprocessing_info)
        setattr(figure_object, "method", method)
        return figure_object

    @check_for_missing_values
    def plot_pca(self, group=None, circle=False):
        """Plot Principal Component Analysis (PCA)

        Args:
            group (str, optional): column in metadata that should be used for coloring. Defaults to None.
            circle (bool, optional): draw circle around each group. Defaults to False.

        Returns:
            plotly.graph_objects._figure.Figure: PCA plot
        """
        dimensionality_reduction = DimensionalityReduction(
            dataset=self, group=group, method="pca", circle=circle
        )
        return dimensionality_reduction.plot


    @check_for_missing_values
    def plot_tsne(self, group=None, circle=False, perplexity=5, n_iter=1000):
        """Plot t-distributed stochastic neighbor embedding (t-SNE)

        Args:
            group (str, optional): column in metadata that should be used for coloring. Defaults to None.
            circle (bool, optional): draw circle around each group. Defaults to False.

        Returns:
            plotly.graph_objects._figure.Figure: t-SNE plot
        """
        dimensionality_reduction = DimensionalityReduction(
            dataset=self,
            group=group,
            method="tsne",
            circle=circle,
            perplexity=perplexity,
            n_iter=n_iter,
        )
        return dimensionality_reduction.plot

    @check_for_missing_values
    def plot_umap(self, group=None, circle=False):
        """Plot Uniform Manifold Approximation and Projection for Dimension Reduction

        Args:
            group (str, optional): column in metadata that should be used for coloring. Defaults to None.
            circle (bool, optional): draw circle around each group. Defaults to False.

        Returns:
            plotly.graph_objects._figure.Figure: UMAP plot
        """
        dimensionality_reduction = DimensionalityReduction(
            dataset=self, group=group, method="umap", circle=circle
        )
        return dimensionality_reduction.plot

    def plot_volcano(
        self,
        group1,
        group2,
        column:str=None,
        method:str="ttest",
        labels:bool=False,
        min_fc:float=1.0,
        alpha:float=0.05,
        draw_line:bool=True,
        perm:int=100, 
        fdr:float=0.05,
        compare_preprocessing_modes:bool=False,
        color_list:list=[]
    ):
        """Plot Volcano Plot

        Args:
            column (str): column name in the metadata file with the two groups to compare
            group1 (str/list): name of group to compare needs to be present in column or list of sample names to compare
            group2 (str/list): name of group to compare needs to be present in column  or list of sample names to compare
            method (str): "anova", "wald", "ttest", "SAM" Defaul ttest.
            labels (bool): Add text labels to significant Proteins, Default False.
            alpha(float,optional): p-value cut off.
            min_fc (float): Minimum fold change.
            draw_line(boolean): whether to draw cut off lines.
            perm(float,optional): number of permutations when using SAM as method. Defaults to 100.
            fdr(float,optional): FDR cut off when using SAM as method. Defaults to 0.05.
            color_list (list): list with ProteinIDs that should be highlighted.
            compare_preprocessing_modes(bool): Will iterate through normalization and imputation modes and return a list of VolcanoPlots in different settings, Default False.


        Returns:
            plotly.graph_objects._figure.Figure: Volcano Plot
        """

        if compare_preprocessing_modes:
            params_for_func = locals()
            results = self._compare_preprocessing_modes(func=VolcanoPlot,params_for_func=params_for_func)
            return results
        
        else:
            volcano_plot = VolcanoPlot(
                dataset=self,
                group1=group1,
                group2=group2,
                column=column,
                method=method,
                labels=labels,
                min_fc=min_fc,
                alpha=alpha,
                draw_line=draw_line,
                perm=perm, 
                fdr=fdr,
                color_list=color_list
            )

            return volcano_plot.plot

    def plot_correlation_matrix(self, method="pearson"):
        """Plot Correlation Matrix

        Args:
            method (str, optional): orrelation coefficient "pearson", "kendall" (Kendall Tau correlation)
            or "spearman" (Spearman rank correlation). Defaults to "pearson".

        Returns:
            plotly.graph_objects._figure.Figure: Correlation matrix
        """
        corr_matrix = self.mat.transpose().corr(method=method)
        plot = px.imshow(corr_matrix)
        return plot

    def plot_sampledistribution(self, method="violin", color=None, log_scale=False):
        """Plot Intensity Distribution for each sample. Either Violin or Boxplot

        Args:
            method (str, optional): Violinplot = "violin", Boxplot = "box". Defaults to "violin".
            color (str, optional): A metadata column used to color the boxes. Defaults to None.
            log_scale (bool, optional): yaxis in logarithmic scale. Defaults to False.

        Returns:
             plotly.graph_objects._figure.Figure: Plotly Sample Distribution Plot
        """

        # create long df
        df = self.mat.unstack().reset_index()
        df.rename(columns={"level_1": self.sample, 0: "Intensity"}, inplace=True)

        if color is not None:
            df = df.merge(self.metadata, how="inner", on=[self.sample])

        if method == "violin":
            fig = px.violin(df, x=self.sample, y="Intensity", color=color)

        elif method == "box":
            fig = px.box(df, x=self.sample, y="Intensity", color=color)

        else:
            raise ValueError(
                f"{method} is not available."
                + "Please select from 'violin' for Violinplot or 'box' for Boxplot."
            )

        if log_scale:
            fig.update_layout(yaxis=dict(type="log"))

        fig = plotly_object(fig)
        fig = self._update_figure_attributes(
            figure_object=fig, plotting_data=df, method=method
        )
        return fig

    def plot_intensity(
        self,
        protein_id,
        group=None,
        subgroups=None,
        method="box",
        add_significance=False,
        log_scale=False,
        compare_preprocessing_modes=False
    ):
        """Plot Intensity of individual Protein/ProteinGroup

        Args:
            ID (str): ProteinGroup ID
            group (str, optional): A metadata column used for grouping. Defaults to None.
            subgroups (list, optional): Select variables from the group column. Defaults to None.
            method (str, optional):  Violinplot = "violin", Boxplot = "box", Scatterplot = "scatter" or "all". Defaults to "box".
            add_significance (bool, optional): add p-value bar, only possible when two groups are compared. Defaults False.
            log_scale (bool, optional): yaxis in logarithmic scale. Defaults to False.

        Returns:
            plotly.graph_objects._figure.Figure: Plotly Plot
        """
        if compare_preprocessing_modes:
            params_for_func = locals()
            results = self._compare_preprocessing_modes(func=IntensityPlot,params_for_func=params_for_func)
            return results
        
        intensity_plot = IntensityPlot(
            dataset = self,
            protein_id=protein_id,
            group=group,
            subgroups=subgroups,
            method=method,
            add_significance=add_significance,
            log_scale=log_scale
        )

        return intensity_plot.plot

    @ignore_warning(UserWarning)
    @check_for_missing_values
    def plot_clustermap(
        self, label_bar=None, only_significant=False, group=None, subgroups=None
    ):
        """Plot a matrix dataset as a hierarchically-clustered heatmap

        Args:
            label_bar (str, optional): column/variable name described in the metadata. Will be plotted as bar above the heatmap to see wheteher groups are clustering together. Defaults to None.. Defaults to None.
            only_significant (bool, optional): performs ANOVA and only signficantly different proteins will be clustered (p<0.05). Defaults to False.
            group (str, optional): group containing subgroups that should be clustered. Defaults to None.
            subgroups (list, optional): variables in group that should be plotted. Defaults to None.

        Returns:
             ClusterGrid: Clustermap
        """

        clustermap = ClusterMap(
            dataset = self,
            label_bar=label_bar,
            only_significant=only_significant,
            group=group,
            subgroups=subgroups
        )
        return  clustermap.plot

    def plot_samplehistograms(self):
        """Plots the Denisty distribution of each sample

        Returns:
            plotly: Plotly Graph Object
        """
        return SampleHistogram(dataset=self).plot()

    @check_for_missing_values
    def plot_dendrogram(
        self, linkagefun=lambda x: scipy.cluster.hierarchy.linkage(x, "complete")
    ):
        """Plot Hierarichical Clustering Dendrogram. This is a wrapper around:
        https://plotly.com/python-api-reference/generated/plotly.figure_factory.create_dendrogram.html

        Args:
            linkagefun (_type_, optional): Function to compute the linkage matrix from
                               the pairwise distance. Defaults to lambdax:scipy.cluster.hierarchy.linkage(x, "complete").

        Raises:
            ValueError: If data contains missing values, is not imputed

        Returns:
            plotly.figure_factory.create_dendrogram: dendrogram Plotly figure object
        """
        # of anova results
        # general of a subset of proteins
        fig = plotly.figure_factory.create_dendrogram(
            self.mat, labels=self.mat.index, linkagefun=linkagefun
        )

        fig = plotly_object(fig)
        fig = self._update_figure_attributes(
            figure_object=fig, plotting_data=self.mat, method="dendrogram"
        )
        return fig

    def plot_imputed_values(self):
        # get coordinates of missing values
        df = self.mat
        s = df.stack(dropna=False)
        missing_values_coordinates = [list(x) for x in s.index[s.isna()]]

        # get all coordinates
        coordinates = list(
            itertools.product(list(self.mat.index), list(self.mat.columns))
        )

        # needs to be speed up
        imputed_values, original_values = [], []
        for coordinate in coordinates:
            coordinate = list(coordinate)
            if coordinate in missing_values_coordinates:
                value = self.mat.loc[coordinate[0], coordinate[1]]
                imputed_values.append(value)
            else:
                original_values.append(value)

        label = ["imputed values"] * len(imputed_values) + ["non imputed values"] * len(
            original_values
        )
        values = imputed_values + original_values

        plot_df = pd.DataFrame(
            list(zip(label, values)), columns=["Imputation", "values"]
        )

        fig = px.histogram(
            plot_df,
            x="values",
            color="Imputation",
            opacity=0.8,
            hover_data=plot_df.columns,
        )

        pass



from alphastats.plots.PlotUtils import PlotUtils, plotly_object
import plotly
import plotly.express as px
import plotly.graph_objects as go
import sklearn
import pandas as pd

# make own alphastats theme
plotly.io.templates["alphastats_colors"] = plotly.graph_objects.layout.Template(
    layout=plotly.graph_objects.Layout(
        paper_bgcolor="rgba(0,0,0,0)",
        plot_bgcolor="rgba(0,0,0,0)",
        colorway=[
            "#009599",
            "#005358",
            "#772173",
            "#B65EAF",  # pink
            "#A73A00",
            "#6490C1",
            "#FF894F",
            "#2B5E8B",
            "#A87F32",
        ],
    )
)

plotly.io.templates.default = "simple_white+alphastats_colors"


class DimensionalityReduction(PlotUtils):
    def __init__(self, dataset, group, method, circle, **kwargs) -> None:
        self.dataset = dataset
        self.method = method
        self.circle = circle
        self.group = group
        self.plot = None

        sample_names, group_color = self._prepare_df()

        if self.method == "pca":
            self._pca()

        elif self.method == "umap":
            self._umap()

        elif self.method == "tsne":
            self._tsne(**kwargs)

        self._plot(sample_names=sample_names, group_color=group_color)

    def _add_circles_to_scatterplot(self, fig):
        # called by _plot_dimensionality_reduction()
        # convert figure to dict and extract information
        fig_dict = fig.to_plotly_json().get("data")
        for group in fig_dict:
            # get coordinates for the group
            x_vector = group.get("x")
            y_vector = group.get("y")
            # get color of the group to color circle in the same color
            group_color = group.get("marker").get("color")
            fig.add_shape(
                type="circle",
                xref="x",
                yref="y",
                x0=min(x_vector),
                y0=min(y_vector),
                x1=max(x_vector),
                y1=max(y_vector),
                opacity=0.2,
                fillcolor=group_color,
                line_color=group_color,
            )
        return fig

    def _prepare_df(self):

        if self.group:
            mat = self.dataset._subset()
            self.dataset.metadata[self.group] = self.dataset.metadata[self.group].apply(
                str
            )
            group_color = self.dataset.metadata[self.group]
            sample_names = self.dataset.metadata[self.dataset.sample].to_list()

        else:
            mat = self.dataset.mat
            group_color = self.group
            sample_names = mat.reset_index(level=0)["index"].to_list()

        mat = mat.fillna(0)
        self.prepared_df = mat

        return sample_names, group_color

    def _pca(self):
        pca = sklearn.decomposition.PCA(n_components=2)
        self.components = pca.fit_transform(self.prepared_df)
        self.labels = {
            "0": "PC 1 (%.2f%%)" % (pca.explained_variance_ratio_[0] * 100),
            "1": "PC 2 (%.2f%%)" % (pca.explained_variance_ratio_[1] * 100),
        }

    def _tsne(self, **kwargs):
        tsne = sklearn.manifold.TSNE(n_components=2, verbose=1, **kwargs)
        self.components = tsne.fit_transform(self.prepared_df)
        self.labels = {
            "0": "Dimension 1",
            "1": "Dimension 2",
        }

    def _umap(self):

        # TODO umap import is reallly buggy 
        try:
            import umap.umap_ as umap
        except ModuleNotFoundError:
            import umap
        
        umap_2d = umap.UMAP(n_components=2, init="random", random_state=0)
        self.components = umap_2d.fit_transform(self.prepared_df)
        self.labels = {
            "0": "",
            "1": "",
        }

    def _plot(self, sample_names, group_color):
        components = pd.DataFrame(self.components)
        components[self.dataset.sample] = sample_names

        fig = px.scatter(
            components,
            x=0,
            y=1,
            labels=self.labels,
            color=group_color,
            hover_data=[components[self.dataset.sample]],
        )

        # rename hover_data_0 to sample
        fig_dict = fig.to_plotly_json()
        data = fig_dict.get("data")

        for count, d in enumerate(data):
            hover = d.get("hovertemplate").replace("hover_data_0", "sample")
            fig_dict["data"][count]["hovertemplate"] = hover
        fig = go.Figure(fig_dict)

        #  save plotting data in figure object
        fig = plotly_object(fig)
        fig = self._update_figure_attributes(
            figure_object=fig,
            plotting_data=pd.DataFrame(components),
            method=self.method,
            preprocessing_info=self.dataset.preprocessing_info
        )

        # draw circles around plotted groups
        if self.circle is True and self.group is not None:
            fig = self._add_circles_to_scatterplot(fig)

        if self.group:
            fig.update_layout(legend_title_text=self.group)

        self.plot = fig



from pyteomics import mztab
from alphastats.loader.BaseLoader import BaseLoader

class mzTabLoader(BaseLoader):
    def __init__(self, file, intensity_column: str="protein_abundance_[sample]", index_column:str="accession"):
        """Load mzTab file. Will add contamination column for further analysis.

        Args:
            file (str): path to mzTab file.
            intensity_column (str, optional):  columns where the intensity of the proteins are given.. Defaults to "protein_abundance_[sample]".
            index_column (str, optional): column indicating the protein groups.  Defaults to "accession".
        """
        self.filter_columns = []
        self.gene_names = None
        self.intensity_column = intensity_column
        self.index_column = index_column
        self.confidence_column = None
        self.evidence_df = None
        self.gene_names = None
        self._load_protein_table(file=file)
        self._add_contamination_column()


    def _load_protein_table(self, file):
        tables = mztab.MzTab(file)
        self.rawinput = tables.protein_table
        self.mztab_metadata = tables.metadata
        self.software = tables.protein_table.search_engine[0]
        


import streamlit as st
import pandas as pd
import base64
from alphastats import __version__

# TODO add logo above the options when issue is closed
# https://github.com/streamlit/streamlit/issues/4984


def sidebar_info(show_logo=True):
    display_sidebar_html_table()
    st.sidebar.markdown("\n\n")
    st.sidebar.markdown("AlphaPeptStats Version " + str(__version__))
    st.sidebar.info(
        "[AlphaPeptStats on GitHub](https://github.com/MannLabs/alphapeptstats)"
    )
    st.sidebar.info(
        "[Documentation](https://alphapeptstats.readthedocs.io/en/latest/index.html)"
    )

    st.sidebar.markdown(
        """ <head><style type ='text/css' > 
    .footer{ position: fixed;     
        text-align: left;    
        bottom: 14px; 
        width: 100%;
    }  
    </style>
    </head>
    <body>
        <div class="footer">Mann Group, 2022</div>
    </body>""",
        unsafe_allow_html=True,
    )



def display_sidebar_html_table():

    if "dataset" not in st.session_state:
        return

    preprocessing_dict = st.session_state.dataset.preprocessing_info

    html_string = (
        "<style>.mytable th, td{ font-size:10px;font-family:Arial, Helvetica, sans-serif;color:#8C878D; border-color:#96D4D4;}</style>"
        "<table class='mytable'>"
    )

    for key, values in preprocessing_dict.items():
        html_string += "<tr><td>" + key + "</td><td>" + str(values) + "</td>" + "</tr>"

    html_string += "</table>"
    st.sidebar.markdown(html_string, unsafe_allow_html=True)


def img_to_bytes(img_path):
    with open(img_path, "rb") as image_file:
        encoded_string = base64.b64encode(image_file.read())
    # img_bytes = Path(img_path).read_bytes()
    # encoded = base64.b64encode(img_bytes).decode()
    return encoded_string.decode()




import logging
import scipy
import numpy as np
import plotly.graph_objects as go
import plotly.express as px

from alphastats.plots.PlotUtils import plotly_object, PlotUtils


class IntensityPlot(PlotUtils):
    def __init__(self,
        dataset,
        protein_id,
        group,
        subgroups,
        method,
        add_significance,
        log_scale,
    ) -> None:
        self.dataset = dataset
        self.protein_id = protein_id
        self.group = group
        self.subgroups = subgroups
        self.method = method
        self.add_significance = add_significance
        self.log_scale = log_scale

        self._prepare_data()
        self._plot()
 

    @staticmethod
    def _add_significance(plot):
        # add sginficance pvalue, and stars to pairwise intensity plot
        plot_dict = plot.to_plotly_json()
        data = plot_dict.get("data")

        if len(data) != 2:
            logging.warning(
                "Signficane can only be estimated when there are two groups plotted."
            )
            return plot

        group1, group2 = data[0]["name"], data[1]["name"]
        y_array1, y_array2 = data[0]["y"], data[1]["y"]
        #  do ttest
        pvalue = scipy.stats.ttest_ind(y_array1, y_array2).pvalue

        pvalue_text = "<i>p=" + str(round(pvalue, 4)) + "</i>"

        if pvalue < 0.001:
            significance_level = "***"
            pvalue_text = "<i>p<0.001</i>"
        
        elif pvalue < 0.01:
            significance_level = "**"
       
        elif pvalue < 0.05:
            significance_level = "*"
       
        else:
            significance_level = "-"

        y_max = np.concatenate((y_array1, y_array2)).max()
        # add connecting bar for pvalue
        plot.add_trace(
            go.Scatter(
                x=[group1, group1, group2, group2],
                y=[y_max * 1.1, y_max * 1.15, y_max * 1.15, y_max * 1.1],
                fill=None,
                mode="lines",
                line=dict(color="rgba(0,0,0,1)", width=1),
                showlegend=False,
            )
        )

        # Add p-values
        plot.add_annotation(
            text=pvalue_text,
            name="p-value",
            xref="paper",
            yref="paper",
            x=0.5,
            y=0.95,
            showarrow=False,
            font=dict(size=12, color="black"),
        )

        plot.add_annotation(
            text=significance_level,
            name="significance",
            xref="paper",
            yref="paper",
            x=0.5,
            y=1.002,
            showarrow=False,
            font=dict(size=12, color="black"),
        )

        plot.update_layout(width=600, height=700)
        return plot
    
    def _prepare_data(self):
        #  TODO use difflib to find similar ProteinId if ProteinGroup is not present
        df = self.dataset.mat[[self.protein_id]].reset_index().rename(columns={"index": self.dataset.sample})
        df = df.merge(self.dataset.metadata, how="inner", on=[self.dataset.sample])

        if self.subgroups is not None:
            df = df[df[self.group].isin(self.subgroups)]

        self.y_label = self.protein_id + " - " + self.dataset.intensity_column.replace("[sample]", "")
        self.prepared_df = df

    def _plot(self):
        if self.method == "violin":
            fig = px.violin(
                self.prepared_df, y=self.protein_id, x=self.group, color=self.group, labels={self.protein_id: self.y_label}
            )

        elif self.method == "box":
            fig = px.box(
                self.prepared_df, y=self.protein_id, x=self.group, color=self.group, labels={self.protein_id: self.y_label}
            )

        elif self.method == "scatter":
            fig = px.scatter(
                 self.prepared_df, y=self.protein_id, x=self.group, color=self.group, labels={self.protein_id: self.y_label}
            )

        elif self.method == "all":
            fig = px.violin(
                 self.prepared_df, y=self.protein_id, x=self.group, color=self.group, labels={self.protein_id: self.y_label},
                 box=True, points="all"
            )

        else:
            raise ValueError(
                f"{self.method} is not available."
                + "Please select from 'violin' for Violinplot, 'box' for Boxplot and 'scatter' for Scatterplot."
            )

        if self.log_scale:
            fig.update_layout(yaxis=dict(type="log"))

        if self.add_significance:
            fig = self._add_significance(fig)

        fig = plotly_object(fig)
        fig = self._update_figure_attributes(
            figure_object=fig, plotting_data= self.prepared_df, preprocessing_info=self.dataset.preprocessing_info,  method=self.method
        )

        self.plot = fig





import streamlit as st
import pandas as pd
import plotly.express as px


@st.cache_data
def convert_df(df, user_session_id = st.session_state.user_session_id):
    return df.to_csv().encode("utf-8")


@st.cache_data
def get_display_matrix(user_session_id = st.session_state.user_session_id):

    processed_df = pd.DataFrame(
        st.session_state.dataset.mat.values,
        index=st.session_state.dataset.mat.index.to_list(),
    ).head(10)

    return processed_df


def display_matrix():

    text = (
        "Normalization: "
        + str(st.session_state.dataset.preprocessing_info["Normalization"])
        + ", Imputation: "
        + str(st.session_state.dataset.preprocessing_info["Imputation"])
        + ", Log2-transformed: "
        + str(st.session_state.dataset.preprocessing_info["Log2-transformed"])
    )

    st.markdown("**DataFrame used for analysis** *preview*")
    st.markdown(text)

    df = get_display_matrix()
    csv = convert_df(st.session_state.dataset.mat)

    st.dataframe(df)

    st.download_button(
        "Download as .csv", csv, "analysis_matrix.csv", "text/csv", key="download-csv"
    )


@st.cache_data
def get_sample_histogram_matrix(user_session_id = st.session_state.user_session_id):
    return st.session_state.dataset.plot_samplehistograms()

@st.cache_data
def get_intensity_distribution_processed(user_session_id = st.session_state.user_session_id):
    return st.session_state.dataset.plot_sampledistribution()



if "dataset" in st.session_state:
    st.markdown("## DataSet overview")

    c1, c2 = st.columns(2)
    
    with c1:

        st.markdown("**Intensity distribution raw data per sample**")
        st.plotly_chart(
            st.session_state.distribution_plot.update_layout(plot_bgcolor="white"),
            use_container_width=True
        )
    
    with c2:

        st.markdown("**Intensity distribution data per sample used for analysis**")
        st.plotly_chart(
            get_intensity_distribution_processed(user_session_id = st.session_state.user_session_id)
                        .update_layout(plot_bgcolor="white"), use_container_width=True
        )
 
    st.plotly_chart(
        get_sample_histogram_matrix(user_session_id = st.session_state.user_session_id)
                    .update_layout(plot_bgcolor="white"), use_container_width=True
    )

    display_matrix()


else:
    st.info("Import Data first")



import scipy
import tqdm
import pandas as pd
from statistics.StatisticUtils import StatisticUtils

class MultiCovaAnalysis(StatisticUtils):
    def __init__(self, dataset, covariates: list, n_permutations: int=3, 
                 fdr: float=0.05, s0: float=0.05, subset: dict=None):
        self.dataset = dataset
        self.covariates = covariates
        self.n_permutations = n_permutations
        self.fdr = fdr
        self.s0 = s0
        self.subset = subset

        self._subset_metadata()
        self._check_covariat_input()
        self._check_na_values()
        self._prepare_matrix()
    
    def _subset_metadata(self):
        if self.subset is not None:
             # dict structure {"column_name": ["group1", "group2"]}
            subset_column = list(self.subset.keys())[0]
            groups = self.subset.get(subset_column)
            self.metadata = self.dataset.metadata[self.dataset.metadata[subset_column].isin(groups)]

        else:
            self.metadata = self.dataset.metadata
    
    def _check_covariat_input(self):
        # check whether covariates in metadata column
        misc_covariates = list(set(self.metadata.columns.to_list()) - set(self.covariates))
        if len(misc_covariates)> 0:
            Warning(f"Covariates: {misc_covariates} are not found in Metadata.")
            self.covariates = filter(lambda i: i not in misc_covariates, self.covariates)
        

    def _check_na_values(self):
        for covariate in self.covariates:
            if self._prepare_matrixmetadata[covariate].isna().any():
                self.covariates.remove(covariate)
                Warning(f"Covariate: {covariate} contains missing values" +
                        f"in metadata and will not be used for analysis.")

    def _prepare_matrix(self):
        transposed = self.dataset.mat.transpose()
        transposed[self.dataset.index_column] = transposed.index
        transposed = transposed.reset_index(drop=True)
        self.transposed = transposed[self.metadata[self.dataset.sample].to_list()]
    
    def calculate(self):
        from alphastats.multicova import multicova
        
        if len(self.covariates) == 0:
            print("Covariates are invalid for analysis.")
            return
        
        res, tlim = multicova.full_regression_analysis(
            quant_data = self.transposed,
            annotation = self.metadata,
            covariates = self.covariates,
            sample_column = self.dataset.sample,
            n_permutations=self.n_permutations,
            fdr=self.fdr, 
            s0=self.s0
        )
        return res
    



